
; ---------------------------------------------------------------------------
; NewTestBed.asm
; ---------------------------------------------------------------------
; Use a Console Screen Format in DOS style.
; Convert it into a CHAR_INFO structure. 
; INVOKE the API: WriteConsoleOutput() to display it all at once . 
; 
; Check for Windows version and display it. 
; Check the CPU brand and display it
; Wait a key to terminate the program, to repeat the test, to save
;      result or to read a previous saved result.
;  
; Only ESC and R/r // S/s // V/v are accepted.
;
; ESC - Key    prompt a dialog box to confirm the exit
; R/r - Key    Rerun the test
; S/s - Key    Save the actual Screen to a POP File with the chosen name
; V/v - Key    Read and Display a previous saved Screen in POP File
;
;
;----------------------------------------------------------------------
; Input file: Precompiled DB generated by Bin2dbx
;----------------------------------------------------------------------
; Author: frktons @ MASM32 forum
; Date: 03/nov/2010. 
;---------------------------------------------------------------------- 
; In this version:
;
; the sizes of the screen and the console are read from the first 4 bytes,
; indicating RowInitial, RowFinal, ColInitial, ColFinal.
; 
; The S/s and V/v options are implemented to Save the actual Screen and
; to Read any previous saved one. Only V/v option so far.
;
; MsgBox used to display messages to the user.
;
; Input File as INCLUDE - not optimized yet for size reduction.
;
; Open File / Save File Dialog to choose Screen File to View / Save
; 
;----------------------------------------------------------------------
; Enjoy - Frank
;----------------------------------------------------------------------------


include \masm32\include\masm32rt.inc



.686
.mmx
.xmm

;----------------------------------------------------------------------------
; PROC PROTO
;----------------------------------------------------------------------------

DisplayFmt     PROTO :DWORD
ClearBuffer    PROTO :DWORD
GetWindowsOS   PROTO :PTR BYTE
DisplayAt      PROTO x:DWORD, y:DWORD, StringBuffer:DWORD, len:DWORD
SetColorTo     PROTO ForeGroundColor:BYTE, BackGroundColor:BYTE
Get_Cpu_Id     PROTO :DWORD
ConvertToWin   PROTO :DWORD, :DWORD
ReadChosenFile PROTO :DWORD 
PopBox         PROTO PtrBoxTitle:DWORD, PtrBoxMessage:DWORD 
SaveScreen     PROTO :DWORD

;----------------------------------------------------------------------------
; Numeric constants used by the program
;----------------------------------------------------------------------------

Zero          EQU 0                  ;initial position on the buffer
Four          EQU 4                  ;number of initial bytes to read
Two           EQU 2                  ;To get the buffer size from chars num

MaxRows       EQU 34                 ;Max Rows of Console
MaxCols       EQU 90                 ;Max Cols of Console

CharNumber    EQU MaxRows*MaxCols
NumCycles     EQU 34*45
BytesToRead   EQU CharNumber*Two

MaxSize       EQU MaxRows*MaxCols*Four  ;Max buffer size

PRODUCT_PROFESSIONAL EQU 30h         ; defined in WinNT.h Windows SDK 7.0A

BrightWhite    EQU 15
BrightYellow   EQU 14
DarkBlue       EQU 1

EscKey         EQU 27
R_Key          EQU 82
r_Key          EQU 114
S_Key          EQU 83
s_Key          EQU 115
V_Key          EQU 86
v_Key          EQU 118

OK             EQU 6
YES            EQU 1
NO             EQU 0


;----------------------------------------------------------------------------
; Initialized data
;----------------------------------------------------------------------------

.data


    ConTitle  db "  ----------     **  New Testbed  ** General Screen File "   
              db "Display Tool  **      ---------- Ver. 0.30 beta", 0   



    AlgoToTest    db   "Conversion of a screen buffer from"
                  db   " DOS to Windows CHAR_INFO structure",0  


    BoxTitle    DB  "Program Message", 0
    BoxMessage  DB  "Sorry Dude,",13,10,"This option is not",13,10
                DB  "Implemented yet, be patient",0 

    BoxMsg0     DB  "Start of Conversion",0                  

    BoxMsg1     DB  "Back from Conversion",0 


      
    ExitMessage DB  "Confirm exit the program ?",0
    ExitTitle   DB  "Exit Program",0   

    CaptionText   BYTE  "Open a Screen File to Display",0
    
    szFilter      BYTE  "POP files *.pop",0,"*.pop",0,0

    BufFileBytes  dd   BytesToRead  ;buffer size to read the POP file




;----------------------------------------------------------------------------
; Include for the Screen to display - precompiled db
;----------------------------------------------------------------------------  

include NewTestBed.inc


    
          
;----------------------------------------------------------------------------
; Uninitialized data
;----------------------------------------------------------------------------         

.data?

    align 16

    ConsoleScreen CHAR_INFO CharNumber dup (<>)

    SecurityArea  db   16 dup (?)

    align 16

    FileScreen    CHAR_INFO CharNumber dup (<>)

    SecurityArea0 db   16 dup (?)

    align 16

    SavedScreen   CHAR_INFO CharNumber dup (<>)

    SecurityArea1 db   16 dup (?)

    align 16

    CleanTestArea CHAR_INFO CharNumber dup (<>)   

    SecurityArea2 db   16 dup (?)

    RowInitialFile    db   ?    
    RowFinalFile      db   ?    
    ColInitialFile    db   ?   
    ColFinalFile      db   ?  

    OddNumber     dd   ?

    ChosenColor   dw   ?
    ActualColor   dw   ? 

    Lenght        dd   ?
        
  

    R1            dw   ?
    R2            dw   ?
    C1            dw   ?
    C2            dw   ?
    

    BufRows       dw   ?          ;buffer height (rows)
    BufCols       dw   ?          ;buffer width (columns)
    BufChars      dd   ?          ;number of chars in the buffer

    FileBuffer2   db   BytesToRead  dup (?)


    PtrFileName   DWORD ?   

    hFile     HANDLE ? 
    cci       CONSOLE_CURSOR_INFO <>        


    wHnd      HANDLE ? 
    rHnd      HANDLE ?

    windowSize SMALL_RECT <>
    bufferSize COORD      <>
    startPoint COORD      <>

    howmany dd ?
    buffer  INPUT_RECORD <>

    OSVERSIONINFOEX_NEW STRUCT
      dwOSVersionInfoSize   DWORD ?
      dwMajorVersion        DWORD ?
      dwMinorVersion        DWORD ?
      dwBuildNumber         DWORD ?
      dwPlatformId          DWORD ?
      szCSDVersion          BYTE  128  dup (?)
      wServicePackMajor     WORD  ?
      wServicePackMinor     WORD  ?
      wSuiteMask            WORD  ?
      wProductType          BYTE  ?
      wReserved             BYTE  ?
    OSVERSIONINFOEX_NEW ENDS

    CTEXT MACRO text:VARARG
      LOCAL TxtName
      .DATA
        TxtName BYTE text,0
      .CODE
      EXITM <ADDR TxtName>
    ENDM

    szOS   BYTE   256 dup (?)

    CpuBuffer BYTE 260 dup (?)
    pbuf   DWORD ?

    ConfirmExit      dd ?
    ButtonClicked    dd ?



.code

start: 

; ---------------------------------------------------------------------------
; The Main Proc shows the program flow.
; ---------------------------------------------------------------------------

Main PROC

    CALL   ConsoleSize 

    INVOKE ClearBuffer, ADDR ConsoleScreen

    INVOKE ClearBuffer, ADDR FileScreen

    INVOKE ClearBuffer, ADDR SavedScreen

    CALL   GetConsole

    CALL   InitProc

    INVOKE ConvertToWin, ADDR ConsoleScreen, ADDR FileBuffer

    INVOKE DisplayFmt, offset ConsoleScreen

    mov    ActualColor, Zero
    
    INVOKE GetWindowsOS, ADDR szOS
    
    INVOKE lstrcat, ADDR szOS, CTEXT(13,10)

    INVOKE SetColorTo, BrightYellow, DarkBlue

    mov    Lenght, len(ADDR szOS)
   
    INVOKE DisplayAt, 7, 1, ADDR szOS, DWORD PTR Lenght

    lea    eax, CpuBuffer

    mov    pbuf, eax

    invoke Get_Cpu_Id, pbuf                ; get the processor string
    mov    pbuf, rv(szMonoSpace,pbuf)      ; format it
    
    mov    Lenght, len(ADDR CpuBuffer)
   
    INVOKE DisplayAt, 7, 2, ADDR CpuBuffer, DWORD PTR Lenght ; display it  

    mov    Lenght, len(ADDR AlgoToTest)
   
    INVOKE DisplayAt, 7, 4, ADDR AlgoToTest, DWORD PTR Lenght

    mov    ConfirmExit, NO

    INVOKE SaveScreen, offset SavedScreen
      
    CALL   AnyKey

    CALL   ShowTheCursor

Finish: 

    exit

Main ENDP

 
; ---------------------------------------------------------------------------
; The size, title and buffer of the Console are set.
; Max Console Size is set to 34 Rows and 90 Cols.
; ---------------------------------------------------------------------------

ConsoleSize PROC

    INVOKE AllocConsole

    INVOKE GetStdHandle, STD_INPUT_HANDLE
    mov rHnd,eax

    INVOKE GetStdHandle, STD_OUTPUT_HANDLE
    mov wHnd,eax 

    INVOKE SetConsoleTitle, ADDR ConTitle
  
    CALL   HideTheCursor

    mov windowSize.Left, Zero
    mov windowSize.Right, (MaxCols -1)
    mov windowSize.Top, Zero
    mov windowSize.Bottom, (MaxRows -1)

    
    mov bufferSize.x, MaxCols
    mov bufferSize.y, MaxRows

    INVOKE SetConsoleWindowInfo, wHnd, TRUE, ADDR windowSize

    INVOKE SetConsoleScreenBufferSize, wHnd, DWORD PTR bufferSize

    ret

ConsoleSize ENDP

; ---------------------------------------------------------------------------
; Reads the Screen File Corner Positions.
; ---------------------------------------------------------------------------
;    The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
; ---------------------------------------------------------------------------
 

GetConsole PROC 


    movzx eax, RowFinal
    mov   R2,  ax
    inc   eax
    sub   al,  RowInitial

    mov   BufRows, ax

    movzx ebx, ColFinal
    mov   C2,  bx    
    inc   ebx
    sub   bl,  ColInitial

    mov   BufCols, bx

    movzx ax, RowInitial  
    mov   R1, ax

    dec   R1
    dec   R2
    movzx ax, ColInitial
    mov   C1, ax 
    dec   C1   
    dec   C2

    mov   ebx, BytesToRead

    mov   BufChars, ebx

    ret 
    
GetConsole ENDP

;----------------------------------------------------------------------------
; The Cursor is made visible
;----------------------------------------------------------------------------

ShowTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,TRUE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
ShowTheCursor ENDP

;----------------------------------------------------------------------------
; The Cursor is made invisible
;----------------------------------------------------------------------------
    


HideTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,FALSE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
HideTheCursor ENDP

;----------------------------------------------------------------------------
; After the Screen File has been read, the parameters to display it are set
;----------------------------------------------------------------------------

InitProc PROC

 
    mov ax, C1
    mov windowSize.Left, ax
    mov ax, C2
    mov windowSize.Right, ax
    mov ax, R1
    mov windowSize.Top, ax
    mov ax, R2
    mov windowSize.Bottom, ax
   
    mov ax, BufRows
    mov bx, BufCols
    
    mov bufferSize.x, bx
    mov bufferSize.y, ax

    mov startPoint.x, Zero
    mov startPoint.y, Zero


    ret

InitProc ENDP

;----------------------------------------------------------------------------
; The Screen File is displayed on the Console 
;----------------------------------------------------------------------------

DisplayFmt PROC szFmtName:DWORD 

    INVOKE WriteConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

DisplayFmt ENDP

;----------------------------------------------------------------------------
; The Screen File is saved for future use 
;----------------------------------------------------------------------------

SaveScreen PROC szFmtName:DWORD 

    INVOKE ReadConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

SaveScreen ENDP

;----------------------------------------------------------------------------
; The program waits for a key pressed
;----------------------------------------------------------------------------
; Returns: key code in buffer.KeyEvent.wVirtualKeyCode WORD size 
; -------------------------------------------------------------------------

AnyKey PROC

again:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz again

    cmp buffer.KeyEvent.bKeyDown,0
    jz again

    mov ax, buffer.KeyEvent.wVirtualKeyCode
    cmp al, EscKey
    jz  ClosePgm

    cmp al, R_Key
    jz  R_Key_Press

    cmp al, r_Key
    jz  R_Key_Press

    cmp al, S_Key
    jz  S_Key_Press

    cmp al, s_Key
    jz  S_Key_Press

    cmp al, V_Key
    jz  V_Key_Press

    cmp al, v_Key
    jz  V_Key_Press

    jmp again

R_Key_Press:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMessage 

    jmp again

S_Key_Press:

    INVOKE PopBox,  OFFSET BoxTitle, OFFSET BoxMessage 

    jmp again

V_Key_Press:

    CALL   ReadScreenFile

WaitingForAKey:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz WaitingForAKey

    cmp buffer.KeyEvent.bKeyDown,0
    jz WaitingForAKey

    INVOKE DisplayFmt, offset SavedScreen

    jmp again


ClosePgm:

    CALL PopBoxExit

    cmp  ConfirmExit, YES

    jnz  again

BackToMain:

    ret 

AnyKey ENDP


; -------------------------------------------------------------------------
; The data read from the file is converted into CHAR_INFO style. 
; The Console Buffer is filled with the content of the File Buffer, padding 
; a zero after each byte making it Windows Console API displayable. 
; -------------------------------------------------------------------------

ConvertToWin PROC DestBuffer:DWORD, SourceBuffer:DWORD


    mov  esi, SourceBuffer
    mov  edi, DestBuffer

    mov  ecx, NumCycles
    xor  eax, eax

    xor  ebx, ebx

    bt   ecx, 0
    jc   IsOdd
    mov  OddNumber, 0
    jmp  NextChar

IsOdd:

    mov OddNumber, 1
    
NextChar:


    mov eax, [esi]
    mov [edi], al
    mov [edi+2], ah

    bswap eax

    mov [edi+Four], ah    
    mov [edi+6], al

    add esi, Four
    add edi, Four*2

    dec ecx
    jnz NextChar 

; --------------------------------------------------------------------------------
;  Verify if the number of characters is odd to manage the extra-bytes
; --------------------------------------------------------------------------------

    bt  OddNumber, 0
    jnc End_cycle

    movzx eax, WORD PTR [esi]
    mov [edi], al
    mov [edi+2], ah   

End_cycle:

    ret 
    
ConvertToWin ENDP

; --------------------------------------------------------------------------------
;  The buffer to be displayed is initially zeroed
; -------------------------------------------------------------------------   

ClearBuffer PROC AddrBuffer:DWORD


    mov edx, AddrBuffer
    lea ecx, [edx + MaxSize]
    xor eax, eax
    movd xmm0, eax
    pshufd xmm0, xmm0, 0
  @@:
    movdqa [edx], xmm0
    add edx, 16
    cmp edx, ecx
    jl @B

    

    ret

ClearBuffer ENDP

; --------------------------------------------------------------------------------
; Check for Windows Version - GregL updated version to detect Win 7
; -------------------------------------------------------------------------

GetWindowsOS PROC pszOS:PTR BYTE

    LOCAL hMod:DWORD
    LOCAL dwType:DWORD
    LOCAL pGNSI:PTR
    LOCAL pGPI:PTR
    LOCAL osvi:OSVERSIONINFOEX_NEW
    LOCAL sysinfo:SYSTEM_INFO
    LOCAL szBuild[16]:BYTE
    LOCAL szBuf[128]:BYTE
    
    INVOKE RtlZeroMemory, ADDR osvi, SIZEOF OSVERSIONINFOEX
    INVOKE RtlZeroMemory, ADDR sysinfo, SIZEOF SYSTEM_INFO
    mov osvi.dwOSVersionInfoSize, SIZEOF OSVERSIONINFOEX
    INVOKE GetVersionEx, ADDR osvi
    ; Call GetNativeSystemInfo if supported or GetSystemInfo otherwise.
    INVOKE GetModuleHandle, CTEXT("kernel32.dll")
    mov hMod, eax
    INVOKE GetProcAddress, hMod, CTEXT("GetNativeSystemInfo")
    mov pGNSI, eax
    .IF pGNSI != NULL
        lea eax, sysinfo
        push eax
        call pGNSI
    .ELSE
        INVOKE GetSystemInfo, ADDR sysinfo
    .ENDIF  
    
    .IF (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) && (osvi.dwMajorVersion > 3)
        ; This program does not support dwMajorVersion less than 4
        INVOKE lstrcpy, pszOS, CTEXT("Microsoft ")
        .IF (osvi.dwMajorVersion == 6)
            .IF (osvi.dwMinorVersion == 0)
                ; Windows Vista or Windows Server 2008
                .IF osvi.wProductType == VER_NT_WORKSTATION
                    INVOKE lstrcat, pszOS, CTEXT("Windows Vista ")
                .ELSE
                    INVOKE lstrcat, pszOS, CTEXT("Windows Server 2008")
                .ENDIF
            .ELSEIF (osvi.dwMinorVersion == 1)
                ; Windows 7 or Windows Server 2008 R2
                .IF osvi.wProductType == VER_NT_WORKSTATION
                    INVOKE lstrcat, pszOS, CTEXT("Windows 7 ")
                .ELSE
                    INVOKE lstrcat, pszOS, CTEXT("Windows Server 2008 R2")
                .ENDIF
            .ENDIF
            
            INVOKE GetProcAddress, hMod, CTEXT("GetProductInfo")
            mov pGPI, eax
            lea eax, dwType
            push eax
            push 0
            push 0
            push osvi.dwMinorVersion
            push osvi.dwMajorVersion
            call pGPI
            .IF dwType == PRODUCT_ULTIMATE
               INVOKE lstrcat, pszOS, CTEXT("Ultimate Edition")
            .ELSEIF dwType == PRODUCT_PROFESSIONAL
               INVOKE lstrcat, pszOS, CTEXT("Professional")               
            .ELSEIF dwType == PRODUCT_HOME_PREMIUM
               INVOKE lstrcat, pszOS, CTEXT("Home Premium Edition")
            .ELSEIF dwType == PRODUCT_HOME_BASIC
               INVOKE lstrcat, pszOS, CTEXT("Home Basic Edition")
            .ELSEIF dwType == PRODUCT_ENTERPRISE
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
            .ELSEIF dwType == PRODUCT_BUSINESS
               INVOKE lstrcat, pszOS, CTEXT("Business Edition")
            .ELSEIF dwType == PRODUCT_STARTER
               INVOKE lstrcat, pszOS, CTEXT("Starter Edition")
            .ELSEIF dwType == PRODUCT_CLUSTER_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Cluster Server Edition")
            .ELSEIF dwType == PRODUCT_DATACENTER_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition")
            .ELSEIF dwType == PRODUCT_DATACENTER_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition (core installation)")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition (core installation)")
            .ELSEIF dwType == PRODUCT_ENTERPRISE_SERVER_IA64
               INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition for Itanium-based Systems")
            .ELSEIF dwType == PRODUCT_SMALLBUSINESS_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Small Business Server")
            .ELSEIF dwType == PRODUCT_SMALLBUSINESS_SERVER_PREMIUM
               INVOKE lstrcat, pszOS, CTEXT("Small Business Server Premium Edition")
            .ELSEIF dwType == PRODUCT_STANDARD_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Standard Edition")
            .ELSEIF dwType == PRODUCT_STANDARD_SERVER_CORE
               INVOKE lstrcat, pszOS, CTEXT("Standard Edition (core installation)")
            .ELSEIF dwType == PRODUCT_WEB_SERVER
               INVOKE lstrcat, pszOS, CTEXT("Web Server Edition")
            .ENDIF    
            .IF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64
                INVOKE lstrcat, pszOS, CTEXT( ", 64-bit" )
            .ELSEIF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL
                INVOKE lstrcat, pszOS, CTEXT(", 32-bit")
            .ENDIF
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 2)
            ; Windows Server 2003    
            INVOKE GetSystemMetrics, SM_SERVERR2
            .IF eax
                INVOKE lstrcat, pszOS, CTEXT("Windows Server 2003 R2, ")
            .ELSEIF osvi.wSuiteMask == VER_SUITE_STORAGE_SERVER    
                INVOKE lstrcat, pszOS, CTEXT( "Windows Storage Server 2003")
            .ELSEIF (osvi.wProductType == VER_NT_WORKSTATION) && (sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
                INVOKE lstrcat, pszOS, CTEXT( "Windows XP Professional x64 Edition")
            .ELSE
                INVOKE lstrcat, pszOS, CTEXT("Windows Server 2003, ")
            .ENDIF
            .IF osvi.wProductType != VER_NT_WORKSTATION
                .IF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64
                    .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                       INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition for Itanium-based Systems")
                    .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                       INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition for Itanium-based Systems")
                    .ENDIF   
                .ENDIF
            .ELSEIF sysinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64
                .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                    INVOKE lstrcat, pszOS, CTEXT("Datacenter x64 Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                    INVOKE lstrcat, pszOS, CTEXT("Enterprise x64 Edition")
                .ELSE 
                    INVOKE lstrcat, pszOS, CTEXT("Standard x64 Edition")
                .ENDIF    
            .ELSE
                .IF osvi.wSuiteMask & VER_SUITE_COMPUTE_SERVER
                    INVOKE lstrcat, pszOS, CTEXT("Compute Cluster Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_DATACENTER
                    INVOKE lstrcat, pszOS, CTEXT("Datacenter Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                    INVOKE lstrcat, pszOS, CTEXT("Enterprise Edition")
                .ELSEIF osvi.wSuiteMask & VER_SUITE_BLADE
                    INVOKE lstrcat, pszOS, CTEXT("Web Edition")
                .ELSE 
                    INVOKE lstrcat, pszOS, CTEXT("Standard Edition")
                .ENDIF    
            .ENDIF
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 1)
            ; Windows XP            
            INVOKE lstrcat, pszOS, CTEXT("Windows XP ")
            .IF osvi.wSuiteMask & VER_SUITE_PERSONAL
                INVOKE lstrcat, pszOS, CTEXT("Home Edition")
            .ELSE 
                INVOKE lstrcat, pszOS, CTEXT("Professional")
            .ENDIF    
        .ELSEIF (osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion == 0)
            ; Windows 2000
            INVOKE lstrcat, pszOS, CTEXT("Windows 2000 ")
             .IF osvi.wProductType == VER_NT_WORKSTATION
                 INVOKE lstrcat, pszOS, CTEXT("Professional" )
             .ELSE
                 .IF osvi.wSuiteMask & VER_SUITE_DATACENTER
                     INVOKE lstrcat, pszOS, CTEXT("Datacenter Server" )
                 .ELSEIF osvi.wSuiteMask & VER_SUITE_ENTERPRISE
                     INVOKE lstrcat, pszOS, CTEXT("Advanced Server")
                 .ELSE 
                     INVOKE lstrcat, pszOS, CTEXT("Server")
                 .ENDIF  
             .ENDIF    
        .ELSEIF osvi.dwMajorVersion == 4
            ; Windows NT 4.0
            INVOKE lstrcat, pszOS, CTEXT("Windows NT 4.0")
        .ENDIF  
        ; Include service pack (if any) and build number.
        INVOKE lstrlen, ADDR osvi.szCSDVersion 
        .IF eax > 0
            INVOKE lstrcat, pszOS, CTEXT(" ")
            INVOKE lstrcat, pszOS, ADDR osvi.szCSDVersion
        .ENDIF
        INVOKE wsprintf, ADDR szBuild, CTEXT(" (build %d)"), osvi.dwBuildNumber
        INVOKE lstrcat, pszOS, ADDR szBuild
        xor eax, eax
    .ELSE
        INVOKE lstrcpy, pszOS, CTEXT("This program does not support Windows versions less than Windows NT 4.0")
        mov eax, 1
    .ENDIF
    ret
GetWindowsOS ENDP

; --------------------------------------------------------------------------------
; Set the attribute colors for next PRINT/DISPLAY instruction
;----------------------------------------------------------------------------


SetColorTo PROC Fore:BYTE, Back:BYTE

    xor  eax, eax

    mov  al, Back
    shl  al, 4
    add  al, Fore
    mov  ChosenColor, ax 

    invoke SetConsoleTextAttribute, wHnd, ChosenColor

    ret

SetColorTo ENDP

; --------------------------------------------------------------------------------
; A string is displayed at Column and Row for the passed Lenght
; ----------------------------------------------------------------------------

DisplayAt PROC x:DWORD, y:DWORD, StringBuffer:DWORD, lenght:DWORD
   
    LOCAL n_of_chars:DWORD
    LOCAL position:COORD
   
    mov eax, x
    mov position.x, ax
    mov eax, y
    mov position.y, ax
   
    invoke SetConsoleCursorPosition, wHnd, DWORD PTR position
    invoke WriteConsole, wHnd, StringBuffer, lenght, ADDR n_of_chars, NULL
   
    ret
    
DisplayAt ENDP


; ----------------------------------------------------------------------------
; --------------------------------------------------------------------------------

;----------------------------------------------------------------------------
; CPU identification code
;----------------------------------------------------------------------------

Get_Cpu_Id PROC pBuffer:DWORD

    LOCAL basicinf  :DWORD
    LOCAL extended  :DWORD

    mov eax, 0                                  ; call basic info function
    cpuid
    mov basicinf, eax

    mov eax, 80000000h                          ; call extended info function
    cpuid
    mov extended, eax

    .if extended == 80000004h || extended == 80000008h
      jmp getID
    .endif

    .if basicinf == 00000001h
      cst pBuffer, "i486 or early Pentium Processor"
      ret
    .elseif basicinf == 00000002h
      cst pBuffer, "Pentium Pro, II or Celeron Processor"
      ret
    .elseif basicinf == 00000003h
      cst pBuffer, "Pentium III Processor"
      ret
    .else
      cst pBuffer, "Cannot Identify x86 Processor"
      ret
    .endif

  getID:

    push ebx
    push esi

    mov esi, pBuffer                    ; load buffer address into ESI
   
    mov eax, 80000002h                  ; extended function 2
    cpuid

    mov [esi],    eax                   ; write results to buffer
    mov [esi+4],  ebx
    mov [esi+8],  ecx
    mov [esi+12], edx

    mov eax, 80000003h                  ; extended function 3
    cpuid

    mov [esi+16], eax                   ; write results to buffer
    mov [esi+20], ebx
    mov [esi+24], ecx
    mov [esi+28], edx

    mov eax, 80000004h                  ; extended function 4
    cpuid

    mov [esi+32], eax                   ; write results to buffer
    mov [esi+36], ebx
    mov [esi+40], ecx
    mov [esi+44], edx

    mov [esi+48], DWORD PTR 0           ; terminate result

    pop esi
    pop ebx

    ret

Get_Cpu_Id ENDP



;----------------------------------------------------------------------------
; Pop UP a Message Box to display some stuff
;----------------------------------------------------------------------------

PopBox PROC PtrBoxTitle:DWORD, PtrBoxMessage:DWORD 


    invoke MessageBox, NULL, PtrBoxMessage,  PtrBoxTitle,
                       MB_OK  OR MB_SYSTEMMODAL OR MB_ICONINFORMATION 

    ret

PopBox ENDP

;----------------------------------------------------------------------------
; Pop UP a Message Box to confirm exiting the program
;----------------------------------------------------------------------------

PopBoxExit PROC


    invoke MessageBox, NULL, offset ExitMessage,  offset ExitTitle,
                       MB_OK  OR MB_SYSTEMMODAL OR MB_ICONQUESTION OR MB_YESNO 

    mov ButtonClicked, eax

    cmp ButtonClicked, OK

    jnz GoBack

    mov ConfirmExit, YES
    
GoBack:  

    ret

PopBoxExit ENDP

;----------------------------------------------------------------------------
; Choose a saved Screen File to Read/Display
;----------------------------------------------------------------------------

ReadScreenFile PROC

    mov PtrFileName, OpenFileDlg (0,0, addr CaptionText, addr szFilter) 

    cmp BYTE PTR [eax], 0 ; test if 1st byte is zero [No selection made]

    je  Finish

    INVOKE ReadChosenFile, DWORD PTR PtrFileName

    INVOKE ConvertToWin, ADDR FileScreen, ADDR FileBuffer2

    INVOKE DisplayFmt, offset FileScreen

Finish:

    ret

ReadScreenFile ENDP  

; ---------------------------------------------------------------------------
; Read the Screen File.
; ---------------------------------------------------------------------------
; 1] The initial 4 bytes contain:
;
;    RowInitial, RowFinal, ColInitial, ColFinal of the Screen File to display
;
; 2] The remaining bytes contain the pairs of Characters/Attributes of
;
;    the Screen File in DOS style.
; ---------------------------------------------------------------------------
 

ReadChosenFile PROC szFmtName:DWORD 

    mov hFile, fopen(szFmtName)

    mov ebx,   fread(hFile, ADDR RowInitialFile, Four)

    mov ebx,   fread(hFile, ADDR FileBuffer2, BytesToRead)

    fclose hFile

    ret 
    
ReadChosenFile ENDP  

;----------------------------------------------------------------------------
; End of program
;----------------------------------------------------------------------------

end start